@@ -2,13 +2,14 @@
 #define GRSEC_DIR		"/etc/grsec"
 #endif
 #define GRLEARN_PATH		"/sbin/grlearn"
+#define GRPAM_PATH		"/sbin/gradm_pam"
 #define GRDEV_PATH		"/dev/grsec"
 #define GR_POLICY_PATH 		GRSEC_DIR "/policy"
 #define GR_PW_PATH 		GRSEC_DIR "/pw"
 #define GR_LEARN_CONFIG_PATH	GRSEC_DIR "/learn_config"
 
-#define GR_VERSION		"2.1.5"
-#define GRADM_VERSION		0x215
+#define GR_VERSION		"2.1.6"
+#define GRADM_VERSION		0x216
 
 #define GR_PWONLY		0
 #define GR_PWANDSUM		1
@@ -31,7 +32,7 @@
 #define PATH_MAX 		4096
 #define MAX_LINE_LEN 		5000
 
-#define MAX_INCLUDE_DEPTH	10
+#define MAX_INCLUDE_DEPTH	20
 #define MAX_NEST_DEPTH		8
 
 #ifndef RLIMIT_LOCKS
@@ -47,12 +48,14 @@ enum {
 	GRADM_MODSEGV 	= 4,
 	GRADM_STATUS 	= 5,
 	GRADM_UNSPROLE 	= 6,
-	GRADM_PASSSET	= 7
+	GRADM_PASSSET	= 7,
+	GRADM_SPROLEPAM = 8
 };
 
 enum {
 	GR_IP_BIND 	= 0x01,
 	GR_IP_CONNECT 	= 0x02,
+	GR_IP_INVERT 	= 0x04,
 };
 
 enum {
@@ -101,7 +104,8 @@ enum {
 	GR_ROLE_GOD 	= 0x0040,
 	GR_ROLE_LEARN 	= 0x0080,
 	GR_ROLE_TPE 	= 0x0100,
-	GR_ROLE_DOMAIN 	= 0x0200
+	GR_ROLE_DOMAIN 	= 0x0200,
+	GR_ROLE_PAM 	= 0x0400,
 };
 
 enum {
@@ -130,7 +134,8 @@ enum {
 	GR_NESTED	= 0x00040000,
 	GR_INHERITLEARN = 0x00080000,
 	GR_PROCFIND	= 0x00100000,
-	GR_POVERRIDE	= 0x00200000
+	GR_POVERRIDE	= 0x00200000,
+	GR_KERNELAUTH	= 0x00400000
 };
 
 /* internal use only.  not to be modified */
@@ -176,7 +181,7 @@ struct ip_acl {
 struct file_acl {
 	char *filename;
 	ino_t inode;
-	gr_dev_t dev;
+	u_int32_t dev;
 	u_int32_t mode;
 
 	struct proc_acl *nested;
@@ -227,7 +232,7 @@ struct role_acl {
 struct proc_acl {
 	char *filename;
 	ino_t inode;
-	gr_dev_t dev;
+	u_int32_t dev;
 	u_int32_t mode;
 	u_int32_t cap_mask;
 	u_int32_t cap_drop;
@@ -291,12 +296,14 @@ struct gr_learn_group_node {
 
 struct gr_learn_file_tmp_node {
 	char *filename;
+	u_int32_t key;
 	u_int32_t mode;
 };
 
 struct gr_learn_user_node {
 	char *rolename;
 	uid_t uid;
+	int multgroups;
 	struct gr_learn_group_node *group;
 	struct gr_hash_struct *hash;
 	struct gr_learn_file_node *subject_list;
@@ -329,7 +336,7 @@ struct gr_pw_entry {
 	unsigned char passwd[GR_PW_LEN];
 	unsigned char sum[GR_SHA_SUM_SIZE];
 	unsigned char salt[GR_SALT_SIZE];
-	gr_dev_t segv_dev;
+	u_int32_t segv_dev;
 	ino_t segv_inode;
 	uid_t segv_uid;
 	u_int16_t mode;
@@ -389,7 +396,7 @@ struct gr_arg {
 	unsigned char sum[GR_SHA_SUM_SIZE];
 	unsigned char sp_role[GR_SPROLE_LEN];
 	struct sprole_pw *sprole_pws;
-	gr_dev_t segv_dev;
+	u_int32_t segv_dev;
 	ino_t segv_inode;
 	uid_t segv_uid;
 	u_int16_t num_sprole_pws;
@@ -405,6 +412,8 @@ struct gr_arg_wrapper {
 char *rlim_table[GR_NLIMITS];
 struct capability_set capability_list[30];
 
+int is_24_kernel;
+
 uid_t special_role_uid;
 
 u_int32_t num_subjects;
