@@ -11,25 +11,14 @@ add_id_transition(struct proc_acl *subject, char *idname, int usergroup, int all
 	int i;
 
 	if (usergroup == GR_ID_USER) {
-		if (allowdeny == GR_ID_ALLOW) {
-			if (subject->user_trans_type & GR_ID_DENY) {
-				fprintf(stderr, "Error on line %lu of %s.  You cannot use "
-					"both user_transition_allow and user_transition_deny.\n"
-					"The RBAC system will not be allowed to be enabled until "
-					"this error is fixed.\n", lineno, current_acl_file);
-				exit(EXIT_FAILURE);
-			}
-			subject->user_trans_type |= GR_ID_ALLOW;
-		} else if (allowdeny == GR_ID_DENY) {
-			if (subject->user_trans_type & GR_ID_ALLOW) {
-				fprintf(stderr, "Error on line %lu of %s.  You cannot use "
-					"both user_transition_allow and user_transition_deny.\n"
-					"The RBAC system will not be allowed to be enabled until "
-					"this error is fixed.\n", lineno, current_acl_file);
-				exit(EXIT_FAILURE);
-			}
-			subject->user_trans_type |= GR_ID_DENY;
+		if ((subject->user_trans_type | allowdeny) == (GR_ID_ALLOW | GR_ID_DENY)) {
+			fprintf(stderr, "Error on line %lu of %s.  You cannot use "
+				"both user_transition_allow and user_transition_deny.\n"
+				"The RBAC system will not be allowed to be enabled until "
+				"this error is fixed.\n", lineno, current_acl_file);
+			exit(EXIT_FAILURE);
 		}
+		subject->user_trans_type |= allowdeny;
 
 		/* dupecheck */
 		for (i = 0; i < subject->user_trans_num; i++)
@@ -55,25 +44,14 @@ add_id_transition(struct proc_acl *subject, char *idname, int usergroup, int all
 		subject->user_transitions = gr_dyn_realloc(subject->user_transitions, subject->user_trans_num * sizeof(uid_t));
 		*(subject->user_transitions + subject->user_trans_num - 1) = pwd->pw_uid;
 	} else if (usergroup == GR_ID_GROUP) {
-		if (allowdeny == GR_ID_ALLOW) {
-			if (subject->group_trans_type & GR_ID_DENY) {
-				fprintf(stderr, "Error on line %lu of %s.  You cannot use "
-					"both group_transition_allow and group_transition_deny.\n"
-					"The RBAC system will not be allowed to be enabled until "
-					"this error is fixed.\n", lineno, current_acl_file);
-				exit(EXIT_FAILURE);
-			}
-			subject->group_trans_type |= GR_ID_ALLOW;
-		} else if (allowdeny == GR_ID_DENY) {
-			if (subject->group_trans_type & GR_ID_ALLOW) {
-				fprintf(stderr, "Error on line %lu of %s.  You cannot use "
-					"both group_transition_allow and group_transition_deny.\n"
-					"The RBAC system will not be allowed to be enabled until "
-					"this error is fixed.\n", lineno, current_acl_file);
-				exit(EXIT_FAILURE);
-			}
-			subject->group_trans_type |= GR_ID_DENY;
+		if ((subject->group_trans_type | allowdeny) == (GR_ID_ALLOW | GR_ID_DENY)) {
+			fprintf(stderr, "Error on line %lu of %s.  You cannot use "
+				"both group_transition_allow and group_transition_deny.\n"
+				"The RBAC system will not be allowed to be enabled until "
+				"this error is fixed.\n", lineno, current_acl_file);
+			exit(EXIT_FAILURE);
 		}
+		subject->group_trans_type |= allowdeny;
 
 		/* dupecheck */
 		for (i = 0; i < subject->group_trans_num; i++)
@@ -109,7 +87,7 @@ is_role_dupe(struct role_acl *role, const char *rolename, const u_int16_t type)
 	struct role_acl *tmp;
 
 	for_each_role(tmp, role)
-	    if ((tmp->roletype & type) && !strcmp(tmp->rolename, rolename))
+	    if ((tmp->roletype & (GR_ROLE_USER | GR_ROLE_GROUP | GR_ROLE_SPECIAL) & type) && !strcmp(tmp->rolename, rolename))
 		return 1;
 
 	return 0;
@@ -369,8 +347,10 @@ add_role_acl(struct role_acl **role, char *rolename, u_int16_t type, int ignore)
 	if (type & GR_ROLE_SPECIAL)
 		add_role_transition(*role,rolename);
 
-	if (type & GR_ROLE_AUTH)
+	if (type & GR_ROLE_AUTH) {
 		add_gradm_acl(*role);
+		add_gradm_pam_acl(*role);
+	}
 	if (!(type & GR_ROLE_SPECIAL))
 		add_grlearn_acl(*role);
 	if (type & GR_ROLE_LEARN)
@@ -487,7 +467,10 @@ display_all_dupes(struct proc_acl *subject, struct file_acl *filp2)
 	for_each_object(tmp, subject)
 	    if (!stat(tmp->filename, &fstat)) {
 		ftmp.inode = fstat.st_ino;
-		ftmp.dev = MKDEV(MAJOR(fstat.st_dev), MINOR(fstat.st_dev));
+		if (is_24_kernel)
+			ftmp.dev = MKDEV_24(MAJOR_24(fstat.st_dev), MINOR_24(fstat.st_dev));
+		else
+			ftmp.dev = MKDEV_26(MAJOR_26(fstat.st_dev), MINOR_26(fstat.st_dev));
 		if (ftmp.inode == filp2->inode && ftmp.dev == filp2->dev)
 			fprintf(stderr, "%s\n", tmp->filename);
 	}
@@ -623,7 +606,10 @@ add_proc_object_acl(struct proc_acl *subject, char *filename,
 	p->filename = filename;
 	p->mode = mode;
 	p->inode = fstat.st_ino;
-	p->dev = MKDEV(MAJOR(fstat.st_dev), MINOR(fstat.st_dev));
+	if (is_24_kernel)
+		p->dev = MKDEV_24(MAJOR_24(fstat.st_dev), MINOR_24(fstat.st_dev));
+	else
+		p->dev = MKDEV_26(MAJOR_26(fstat.st_dev), MINOR_26(fstat.st_dev));
 
 	if (type & GR_FLEARN) {
 		struct file_acl *tmp;
@@ -724,7 +710,10 @@ add_proc_subject_acl(struct role_acl *role, char *filename, u_int32_t mode, int
 	p->filename = filename;
 	p->mode = mode;
 
-	p->dev = MKDEV(MAJOR(fstat.st_dev), MINOR(fstat.st_dev));
+	if (is_24_kernel)
+		p->dev = MKDEV_24(MAJOR_24(fstat.st_dev), MINOR_24(fstat.st_dev));
+	else
+		p->dev = MKDEV_26(MAJOR_26(fstat.st_dev), MINOR_26(fstat.st_dev));
 	p->inode = fstat.st_ino;
 
 	if (!(flag & GR_FFAKE) && (p2 = is_proc_subject_dupe(role, p))) {
@@ -785,9 +774,26 @@ role_mode_conv(const char *mode)
 		case 'T':
 			retmode |= GR_ROLE_TPE;
 			break;
+		
+		case 'P':
+			retmode |= GR_ROLE_PAM;
+			break;
 		}
 	}
 
+	if (
+	    ((retmode & (GR_ROLE_NOPW | GR_ROLE_AUTH | GR_ROLE_PAM)) ==
+	    (GR_ROLE_NOPW | GR_ROLE_AUTH | GR_ROLE_PAM)) ||
+	    ((retmode & (GR_ROLE_NOPW | GR_ROLE_AUTH | GR_ROLE_PAM)) ==
+	     (GR_ROLE_NOPW | GR_ROLE_AUTH)) ||
+	    ((retmode & (GR_ROLE_NOPW | GR_ROLE_AUTH | GR_ROLE_PAM)) ==
+	     (GR_ROLE_AUTH | GR_ROLE_PAM)) ||
+	    ((retmode & (GR_ROLE_NOPW | GR_ROLE_AUTH | GR_ROLE_PAM)) ==
+	     (GR_ROLE_NOPW | GR_ROLE_PAM))) {
+		fprintf(stderr, "Error on line %lu of %s.  The role mode must contain only one of the auth, noauth, and pamauth modes.\n", lineno, current_acl_file);
+		exit(EXIT_FAILURE);
+	}
+
 	if (retmode & GR_ROLE_SPECIAL &&
 	    retmode & (GR_ROLE_USER | GR_ROLE_GROUP)) {
 		fprintf(stderr, "Error on line %lu of %s.  The role mode must be either "
@@ -889,6 +895,9 @@ proc_subject_mode_conv(const char *mode)
 		case 'i':
 			retmode |= GR_INHERITLEARN;
 			break;
+		case 'a':
+			retmode |= GR_KERNELAUTH;
+			break;
 		default:
 			fprintf(stderr, "Invalid proc subject mode "
 				"\'%c\' found on line %lu "
@@ -1002,7 +1011,6 @@ parse_acls(void)
 	gradmin = open_acl_file(GR_POLICY_PATH);
 	change_current_acl_file(GR_POLICY_PATH);
 	gradmparse();
-	fclose(gradmin);
 
 	add_kernel_acl();
 
@@ -1026,7 +1034,7 @@ setup_special_roles(struct gr_arg *grarg)
 
 	for_each_role(rtmp, current_role) {
 		if (rtmp->roletype & GR_ROLE_SPECIAL &&
-		    !(rtmp->roletype & GR_ROLE_NOPW)) {
+		    !(rtmp->roletype & (GR_ROLE_NOPW | GR_ROLE_PAM))) {
 			strncpy(entry.rolename, rtmp->rolename, GR_SPROLE_LEN);
 			entry.rolename[GR_SPROLE_LEN - 1] = '\0';
 			if (!read_saltandpass
@@ -1067,7 +1075,7 @@ conv_user_to_kernel(struct gr_pw_entry *entry)
 	for_each_role(rtmp, current_role) {
 		racls++;
 		if (rtmp->roletype & GR_ROLE_SPECIAL &&
-		    !(rtmp->roletype & GR_ROLE_NOPW))
+		    !(rtmp->roletype & (GR_ROLE_NOPW | GR_ROLE_PAM)))
 			sproles++;
 	}
 
@@ -1079,7 +1087,7 @@ conv_user_to_kernel(struct gr_pw_entry *entry)
 	     (struct gr_arg_wrapper *) calloc(1, sizeof (struct gr_arg_wrapper))) == NULL)
 		failure("calloc");
 
-	wrapper->version = GRADM_VERSION | (KERNVER << 16);
+	wrapper->version = GRADM_VERSION;
 	wrapper->size = sizeof(struct gr_arg);
 	wrapper->arg = retarg;
 
